@@ -1,16 +1,20 @@
-from typing import Optional, Tuple, List
+from datetime import datetime
+from typing import Optional, Tuple, List, Union
 
 from flask_security import current_user
-from sqlalchemy.orm import Session
+import pandas as pd
 from sqlalchemy.engine import Row
-
 from sqlalchemy.ext.hybrid import hybrid_method
 from sqlalchemy.sql.expression import func
 from sqlalchemy.ext.mutable import MutableDict
 from sqlalchemy.schema import UniqueConstraint
 
 from flexmeasures.data import db
+from flexmeasures.data.models.annotations import Annotation
+from flexmeasures.data.models.data_sources import DataSource
+from flexmeasures.data.models.parsing_utils import parse_source_arg
 from flexmeasures.data.models.user import User
+from flexmeasures.data.queries.annotations import query_asset_annotations
 from flexmeasures.auth.policy import AuthModelMixin, EVERY_LOGGED_IN_USER
 from flexmeasures.utils import geo_utils
 
@@ -47,6 +51,11 @@ class GenericAsset(db.Model, AuthModelMixin):
         foreign_keys=[generic_asset_type_id],
         backref=db.backref("generic_assets", lazy=True),
     )
+    annotations = db.relationship(
+        "Annotation",
+        secondary="annotations_assets",
+        backref=db.backref("assets", lazy="dynamic"),
+    )
 
     __table_args__ = (
         UniqueConstraint(
@@ -59,11 +68,11 @@ class GenericAsset(db.Model, AuthModelMixin):
     def __acl__(self):
         """
         All logged-in users can read if the asset is public.
-        Within same account, everyone can read and update.
-        Creation and deletion are left to account admins.
+        Within same account, everyone can create, read and update.
+        Deletion is left to account admins.
         """
         return {
-            "create-children": (f"account:{self.account_id}", "role:account-admin"),
+            "create-children": f"account:{self.account_id}",
             "read": f"account:{self.account_id}"
             if self.account_id is not None
             else EVERY_LOGGED_IN_USER,
@@ -72,7 +81,7 @@ class GenericAsset(db.Model, AuthModelMixin):
         }
 
     def __repr__(self):
-        return "<GenericAsset %s:%r (%s)>" % (
+        return "<GenericAsset %s: %r (%s)>" % (
             self.id,
             self.name,
             self.generic_asset_type.name,
@@ -164,9 +173,78 @@ class GenericAsset(db.Model, AuthModelMixin):
 
     @property
     def has_energy_sensors(self) -> bool:
-        """True if at least one power energy is attached"""
+        """True if at least one energy sensor is attached"""
         return any([s.measures_energy for s in self.sensors])
 
+    def add_annotations(
+        self,
+        df: pd.DataFrame,
+        annotation_type: str,
+        commit_transaction: bool = False,
+    ):
+        """Add a data frame describing annotations to the database, and assign the annotations to this asset."""
+        annotations = Annotation.add(df, annotation_type=annotation_type)
+        self.annotations += annotations
+        db.session.add(self)
+        if commit_transaction:
+            db.session.commit()
+
+    def search_annotations(
+        self,
+        annotation_starts_after: Optional[datetime] = None,
+        annotation_ends_before: Optional[datetime] = None,
+        source: Optional[
+            Union[DataSource, List[DataSource], int, List[int], str, List[str]]
+        ] = None,
+        annotation_type: str = None,
+        include_account_annotations: bool = False,
+        as_frame: bool = False,
+    ) -> Union[List[Annotation], pd.DataFrame]:
+        """Return annotations assigned to this asset, and optionally, also those assigned to the asset's account."""
+        parsed_sources = parse_source_arg(source)
+        annotations = query_asset_annotations(
+            asset_id=self.id,
+            annotation_starts_after=annotation_starts_after,
+            annotation_ends_before=annotation_ends_before,
+            sources=parsed_sources,
+            annotation_type=annotation_type,
+        ).all()
+        if include_account_annotations:
+            annotations += self.owner.search_annotations(
+                annotation_starts_before=annotation_starts_after,
+                annotation_ends_before=annotation_ends_before,
+                source=source,
+            )
+
+        if as_frame:
+            return pd.DataFrame(
+                [
+                    [a.start, a.end, a.belief_time, a.source, a.type, a.content]
+                    for a in annotations
+                ],
+                columns=["start", "end", "belief_time", "source", "type", "content"],
+            )
+        return annotations
+
+    def count_annotations(
+        self,
+        annotation_starts_after: Optional[datetime] = None,
+        annotation_ends_before: Optional[datetime] = None,
+        source: Optional[
+            Union[DataSource, List[DataSource], int, List[int], str, List[str]]
+        ] = None,
+        annotation_type: str = None,
+    ) -> int:
+        """Count the number of annotations assigned to this asset."""
+        parsed_sources = parse_source_arg(source)
+        return query_asset_annotations(
+            asset_id=self.id,
+            annotation_starts_after=annotation_starts_after,
+            annotation_ends_before=annotation_ends_before,
+            sources=parsed_sources,
+            annotation_type=annotation_type,
+        ).count()
+
 
 def create_generic_asset(generic_asset_type: str, **kwargs) -> GenericAsset:
     """Create a GenericAsset and assigns it an id.
@@ -213,9 +291,7 @@ def assets_share_location(assets: List[GenericAsset]) -> bool:
     return all([a.location == assets[0].location for a in assets])
 
 
-def get_center_location_of_assets(
-    db: Session, user: Optional[User]
-) -> Tuple[float, float]:
+def get_center_location_of_assets(user: Optional[User]) -> Tuple[float, float]:
     """
     Find the center position between all generic assets of the user's account.
     """
