@@ -1,12 +1,12 @@
-from flask_security import roles_accepted
 from flask_json import as_json
 from bokeh.embed import json_item
 from marshmallow import Schema, fields, validate
 from webargs.flaskparser import use_args
 
+from flexmeasures.auth.decorators import account_roles_accepted
 from flexmeasures.api.v2_0 import flexmeasures_api as flexmeasures_api_v2_0
 from flexmeasures.api.v2_0.routes import v2_0_service_listing
-from flexmeasures.api.common.schemas.times import DurationField
+from flexmeasures.data.schemas.times import DurationField
 from flexmeasures.data.queries.analytics import get_power_data
 from flexmeasures.ui.views.analytics import make_power_figure
 
@@ -14,12 +14,9 @@ from flexmeasures.ui.views.analytics import make_power_figure
 """
 An endpoint to get a power chart.
 
-This will grow to become code for more charts eventually.
-The plan is to separate charts specs from the actual data later,
-and to switch to Altair.
-
-For now, we'll keep this endpoint here, with route and implementation in the same file.
-When we move forward, we'll review the architecture.
+This chart view is considered legacy now. See flexmeasures/data/models/charts for our new approach
+to support charts per API, using Altair and the Sensor ID.
+It also uses the old data model.
 """
 
 
@@ -49,7 +46,7 @@ class ChartRequestSchema(Schema):
 
 
 @flexmeasures_api_v2_0.route("/charts/power", methods=["GET"])
-@roles_accepted("admin", "Prosumer")
+@account_roles_accepted("Prosumer")
 @use_args(ChartRequestSchema(), location="querystring")
 @as_json
 def get_power_chart(chart_request):
