@@ -11,60 +11,55 @@ from flexmeasures.utils.time_utils import get_first_day_of_next_month
 
 
 """
-Functionality to support parsing and building USEF's EA1 addressing scheme [1],
-which is mostly taken from IETF RFC 3720 [2]:
-
-This is the complete structure of an EA1 address:
-
-ea1.{date code}.{reversed domain name}:{locally unique string}
-
-for example "ea1.2021-01.io.flexmeasures.company:sensor14"
-
-- "ea1" is a constant, indicating this is a type 1 USEF entity address
-- The date code "must be a date during which the naming authority owned
-    the domain name used in this format, and should be the first month in which the domain name was
-    owned by this naming authority at 00:01 GMT of the first day of the month.
-- The reversed domain name is taken from the naming authority
-    (person or organization) creating this entity address
-- The locally unique string can be used for local purposes, and FlexMeasures
-  uses it to identify the resource (more information in parse_entity_address).
-  Fields in the locally unique string are separated by colons, see for other examples
-  IETF RFC 3721, page 6 [3].
-  ([2] says it's possible to use dashes, dots or colons ― dashes and dots might come up in
-  latitude/longitude coordinates of sensors)
-
-TODO: This needs to be in the FlexMeasures documentation.
+Functionality to support parsing and building Entity Addresses as defined by USEF [1].
+See our documentation for more details.
 
 [1] https://www.usef.energy/app/uploads/2020/01/USEF-Flex-Trading-Protocol-Specifications-1.01.pdf
-[2] https://tools.ietf.org/html/rfc3720
-[3] https://tools.ietf.org/html/rfc3721
 """
 
 
 ADDR_SCHEME = "ea1"
+FM1_ADDR_SCHEME = "fm1"
+FM0_ADDR_SCHEME = "fm0"
 
 
 class EntityAddressException(Exception):
     pass
 
 
+def get_host() -> str:
+    """Get host from the context of the request.
+
+    Strips off www. but keeps subdomains.
+    Can be localhost, too.
+    """
+    if has_request_context():
+        host = urlparse(request.url).netloc.lstrip("www.")
+        if host[:9] != "127.0.0.1":
+            return host
+    # Assume localhost (for CLI/tests/simulations)
+    return "localhost"
+
+
 def build_entity_address(
-    entity_info: dict, entity_type: str, host: Optional[str] = None
+    entity_info: dict,
+    entity_type: str,
+    host: Optional[str] = None,
+    fm_scheme: str = FM1_ADDR_SCHEME,
 ) -> str:
     """
     Build an entity address.
 
+    fm1 type entity address should use entity_info["sensor_id"]
+    todo: implement entity addresses for actuators with entity_info["actuator_id"] (first ensuring globally unique ids across sensors and actuators)
+
     If the host is not given, it is attempted to be taken from the request.
     entity_info is expected to contain the required fields for the custom string.
 
     Returns the address as string.
     """
     if host is None:
-        if has_request_context():
-            host = urlparse(request.url).netloc
-        else:
-            # Assume localhost (for CLI/tests/simulations)
-            host = "localhost"
+        host = get_host()
 
     def build_field(field: str, required: bool = True):
         if required and field not in entity_info:
@@ -73,13 +68,21 @@ def build_entity_address(
             )
         if field not in entity_info:
             return ""
-        return f":{entity_info[field]}"
+        return f"{entity_info[field]}:"
 
-    if entity_type == "connection":
+    if fm_scheme == FM1_ADDR_SCHEME:  # and entity_type == "sensor":
+        locally_unique_str = f"{build_field('sensor_id')}"
+    # elif fm_scheme == FM1_ADDR_SCHEME and entity_type == "actuator":
+    #     locally_unique_str = f"{build_field('actuator_id')}"
+    elif fm_scheme != FM0_ADDR_SCHEME:
+        raise EntityAddressException(
+            f"Unrecognized FlexMeasures scheme for entity addresses: {fm_scheme}"
+        )
+    elif entity_type == "connection":
         locally_unique_str = (
             f"{build_field('owner_id', required=False)}{build_field('asset_id')}"
         )
-    elif entity_type == "sensor":
+    elif entity_type == "weather_sensor":
         locally_unique_str = f"{build_field('weather_sensor_type_name')}{build_field('latitude')}{build_field('longitude')}"
     elif entity_type == "market":
         locally_unique_str = f"{build_field('market_name')}"
@@ -87,75 +90,140 @@ def build_entity_address(
         locally_unique_str = f"{build_field('owner_id', required=False)}{build_field('asset_id')}{build_field('event_id')}{build_field('event_type')}"
     else:
         raise EntityAddressException(f"Unrecognized entity type: {entity_type}")
-    return build_ea_scheme_and_naming_authority(host) + locally_unique_str
+    return (
+        build_ea_scheme_and_naming_authority(host)
+        + ":"
+        + fm_scheme
+        + "."
+        + locally_unique_str.rstrip(":")
+    )
 
 
-def parse_entity_address(entity_address: str, entity_type: str) -> dict:
+def parse_entity_address(  # noqa: C901
+    entity_address: str,
+    entity_type: str,
+    fm_scheme: str = FM1_ADDR_SCHEME,
+) -> dict:
     """
-    Parses a generic asset name into an info dict.
-
-    The entity_address must be a valid type 1 USEF entity address.
-    That is, it must follow the EA1 addressing scheme recommended by USEF.
-    In addition, FlexMeasures expects the identifying string to contain information in
-    a certain structure.
-
-    For example:
-
-        connection = ea1.2018-06.localhost:40:30
-        connection = ea1.2018-06.io.flexmeasures:<owner_id>:<asset_id>
-        sensor = ea1.2018-06.io.flexmeasures:temperature:52:73.0
-        sensor = ea1.2018-06.io.flexmeasures:<sensor_type>:<latitude>:<longitude>
-        market = ea1.2018-06.io.flexmeasures:epex_da
-        market = ea1.2018-06.io.flexmeasures:<market_name>
-        event = ea1.2018-06.io.flexmeasures:40:30:302:soc
-        event = ea1.2018-06.io.flexmeasures:<owner_id>:<asset_id>:<event_id>:<event_type>
+    Parses an entity address into an info dict.
 
     Returns a dictionary with scheme, naming_authority and various other fields,
-    depending on the entity type (see examples above).
-    Returns None if entity type is unknown or entity_address is not parseable.
+    depending on the entity type and FlexMeasures scheme (see examples above).
+    Returns None if entity type is unknown or entity_address is not parse-able.
     We recommend to `return invalid_domain()` in that case.
+
+    Examples for the fm1 scheme:
+
+        sensor = ea1.2021-01.io.flexmeasures:fm1.42
+        sensor = ea1.2021-01.io.flexmeasures:fm1.<sensor_id>
+        connection = ea1.2021-01.io.flexmeasures:fm1.<sensor_id>
+        market = ea1.2021-01.io.flexmeasures:fm1.<sensor_id>
+        weather_station = ea1.2021-01.io.flexmeasures:fm1.<sensor_id>
+        todo: UDI events are not yet modelled in the fm1 scheme, but will probably be ea1.2021-01.io.flexmeasures:fm1.<actuator_id>
+
+    Examples for the fm0 scheme:
+
+        connection = ea1.2021-01.localhost:fm0.40:30
+        connection = ea1.2021-01.io.flexmeasures:fm0.<owner_id>:<asset_id>
+        weather_sensor = ea1.2021-01.io.flexmeasures:fm0.temperature:52:73.0
+        weather_sensor = ea1.2021-01.io.flexmeasures:fm0.<sensor_type>:<latitude>:<longitude>
+        market = ea1.2021-01.io.flexmeasures:fm0.epex_da
+        market = ea1.2021-01.io.flexmeasures:fm0.<market_name>
+        event = ea1.2021-01.io.flexmeasures:fm0.40:30:302:soc
+        event = ea1.2021-01.io.flexmeasures:fm0.<owner_id>:<asset_id>:<event_id>:<event_type>
+
+    For the fm0 scheme, the 'fm0.' part is optional, for backwards compatibility.
     """
-    # we can rigidly test the start
+
+    # Check the scheme and naming authority date
     if not entity_address.startswith(ADDR_SCHEME):
         raise EntityAddressException(
             f"A valid type 1 USEF entity address starts with '{ADDR_SCHEME}', please review {entity_address}"
         )
-    date_regex = r"[0-9]{4}-[0-9]{2}"
+    date_regex = r"([0-9]{4})-(0[1-9]|1[012])"
     if not re.search(fr"^{date_regex}$", entity_address[4:11]):
         raise EntityAddressException(
-            f"After '{ADDR_SCHEME}.', a date spec of the format {date_regex} is expected."
+            f"After '{ADDR_SCHEME}.', a date specification of the format {date_regex} is expected."
         )
-    # Also the entity type
-    if entity_type not in ("connection", "sensor", "market", "event"):
+
+    # Check the entity type
+    if entity_type not in ("sensor", "connection", "weather_sensor", "market", "event"):
         raise EntityAddressException(f"Unrecognized entity type: {entity_type}")
 
-    if entity_type == "connection":
+    def validate_ea_for_fm_scheme(ea: dict, fm_scheme: str):
+        if "fm_scheme" not in ea:
+            # Backwards compatibility: assume fm0 if fm_scheme is not specified
+            ea["fm_scheme"] = FM0_ADDR_SCHEME
+        scheme = ea["scheme"]
+        naming_authority = ea["naming_authority"]
+        if ea["fm_scheme"] != fm_scheme:
+            raise EntityAddressException(
+                f"A valid type {fm_scheme[2:]} FlexMeasures entity address starts with '{scheme}.{naming_authority}:{fm_scheme}', please review {entity_address}"
+            )
+
+    if fm_scheme == FM1_ADDR_SCHEME:
+
+        # Check the FlexMeasures scheme
+        if entity_address.split(":")[1][: len(fm_scheme) + 1] != FM1_ADDR_SCHEME + ".":
+            raise EntityAddressException(
+                f"A valid type {fm_scheme[2:]} FlexMeasures entity address starts with '{build_ea_scheme_and_naming_authority(get_host())}:{fm_scheme}.', please review {entity_address}"
+            )
+
         match = re.search(
             r"^"
             r"(?P<scheme>.+)\."
             fr"(?P<naming_authority>{date_regex}\.[^:]+)"  # everything until the colon (no port)
             r":"
+            r"((?P<fm_scheme>.+)\.)"
+            r"(?P<sensor_id>\d+)"
+            r"$",
+            entity_address,
+        )
+        if match is None:
+            raise EntityAddressException(
+                f"Could not parse {entity_type} {entity_address}."
+            )
+        value_types = {
+            "scheme": str,
+            "naming_authority": str,
+            "fm_scheme": str,
+            "sensor_id": int,
+        }
+    elif fm_scheme != FM0_ADDR_SCHEME:
+        raise EntityAddressException(
+            f"Unrecognized FlexMeasures scheme for entity addresses: {fm_scheme}"
+        )
+    elif entity_type == "connection":
+        match = re.search(
+            r"^"
+            r"(?P<scheme>.+)\."
+            fr"(?P<naming_authority>{date_regex}\.[^:]+)"  # everything until the colon (no port)
+            r":"
+            r"((?P<fm_scheme>.+)\.)*"  # for backwards compatibility, missing fm_scheme is interpreted as fm0
             r"((?P<owner_id>\d+):)*"  # owner id is optional
             r"(?P<asset_id>\d+)"
             r"$",
             entity_address,
         )
-        if match:
-            value_types = {
-                "scheme": str,
-                "naming_authority": str,
-                "owner_id": int,
-                "asset_id": int,
-            }
-            return _typed_regex_results(match, value_types)
-    elif entity_type == "sensor":
+        if match is None:
+            raise EntityAddressException(
+                f"Could not parse {entity_type} {entity_address}."
+            )
+        value_types = {
+            "scheme": str,
+            "naming_authority": str,
+            "owner_id": int,
+            "asset_id": int,
+        }
+    elif entity_type == "weather_sensor":
         match = re.search(
             r"^"
             r"(?P<scheme>.+)"
             r"\."
             fr"(?P<naming_authority>{date_regex}\.[^:]+)"
             r":"
-            r"(?=[a-zA-Z])(?P<weather_sensor_type_name>[\w]+)"  # should start with at least one letter
+            r"((?P<fm_scheme>.+)\.)*"  # for backwards compatibility, missing fm_scheme is interpreted as fm0
+            r"(?=[a-zA-Z])(?P<weather_sensor_type_name>[\w\s]+)"  # should start with at least one letter
             r":"
             r"(?P<latitude>\-?\d+(\.\d+)?)"
             r":"
@@ -163,15 +231,17 @@ def parse_entity_address(entity_address: str, entity_type: str) -> dict:
             r"$",
             entity_address,
         )
-        if match:
-            value_types = {
-                "scheme": str,
-                "naming_authority": str,
-                "weather_sensor_type_name": str,
-                "latitude": float,
-                "longitude": float,
-            }
-            return _typed_regex_results(match, value_types)
+        if match is None:
+            raise EntityAddressException(
+                f"Could not parse {entity_type} {entity_address}."
+            )
+        value_types = {
+            "scheme": str,
+            "naming_authority": str,
+            "weather_sensor_type_name": str,
+            "latitude": float,
+            "longitude": float,
+        }
     elif entity_type == "market":
         match = re.search(
             r"^"
@@ -179,13 +249,16 @@ def parse_entity_address(entity_address: str, entity_type: str) -> dict:
             r"\."
             fr"(?P<naming_authority>{date_regex}\.[^:]+)"
             r":"
+            r"((?P<fm_scheme>.+)\.)*"  # for backwards compatibility, missing fm_scheme is interpreted as fm0
             r"(?=[a-zA-Z])(?P<market_name>[\w]+)"  # should start with at least one letter
             r"$",
             entity_address,
         )
-        if match:
-            value_types = {"scheme": str, "naming_authority": str, "market_name": str}
-            return _typed_regex_results(match, value_types)
+        if match is None:
+            raise EntityAddressException(
+                f"Could not parse {entity_type} {entity_address}."
+            )
+        value_types = {"scheme": str, "naming_authority": str, "market_name": str}
     elif entity_type == "event":
         match = re.search(
             r"^"
@@ -193,6 +266,7 @@ def parse_entity_address(entity_address: str, entity_type: str) -> dict:
             r"\."
             fr"(?P<naming_authority>{date_regex}\.[^:]+)"
             r":"
+            r"((?P<fm_scheme>.+)\.)*"  # for backwards compatibility, missing fm_scheme is interpreted as fm0
             r"((?P<owner_id>\d+):)*"  # owner id is optional
             r"(?P<asset_id>\d+)"
             r":"
@@ -202,19 +276,25 @@ def parse_entity_address(entity_address: str, entity_type: str) -> dict:
             r"$",
             entity_address,
         )
-        if match:
-            value_types = {
-                "scheme": str,
-                "naming_authority": str,
-                "owner_id": int,
-                "asset_id": int,
-                "event_id": int,
-                "event_type": str,
-            }
-            return _typed_regex_results(match, value_types)
+        if match is None:
+            raise EntityAddressException(
+                f"Could not parse {entity_type} {entity_address}."
+            )
+        value_types = {
+            "scheme": str,
+            "naming_authority": str,
+            "owner_id": int,
+            "asset_id": int,
+            "event_id": int,
+            "event_type": str,
+        }
+    else:
+        # Finally, we simply raise without precise information what went wrong
+        raise EntityAddressException(f"Could not parse {entity_address}.")
 
-    # Finally, we simply raise without precise information what went wrong
-    raise EntityAddressException(f"Could not parse {entity_address}.")
+    ea = _typed_regex_results(match, value_types)
+    validate_ea_for_fm_scheme(ea, fm_scheme)
+    return ea
 
 
 def build_ea_scheme_and_naming_authority(
@@ -236,14 +316,14 @@ def build_ea_scheme_and_naming_authority(
                 [domain_parts.subdomain, domain_parts.domain, domain_parts.suffix],
             )
         )
-        if config_var_domain_key in current_app.config.get(
+        if domain_parts.domain in ("localhost", "127.0.0.1"):
+            host_auth_start_month = get_first_day_of_next_month().strftime("%Y-%m")
+        elif config_var_domain_key in current_app.config.get(
             "FLEXMEASURES_HOSTS_AND_AUTH_START", {}
         ):
             host_auth_start_month = current_app.config.get(
                 "FLEXMEASURES_HOSTS_AND_AUTH_START", {}
             )[config_var_domain_key]
-        elif domain_parts.domain in ("localhost", "127.0.0.1"):
-            host_auth_start_month = get_first_day_of_next_month().strftime("%Y-%m")
         else:
             raise Exception(
                 f"Could not find out when authority for {config_var_domain_key} started. Is FLEXMEASURES_HOSTS_AND_AUTH_START configured for it?"
