@@ -1,7 +1,10 @@
+from __future__ import annotations
 import re
 
+import inflect
 import inflection
 
+p = inflect.engine()
 
 # Give the inflection module some help for our domain
 inflection.UNCOUNTABLES.add("solar")
@@ -11,7 +14,7 @@ ACRONYMS = ["EVSE"]
 
 
 def capitalize(x: str, lower_case_remainder: bool = False) -> str:
-    """ Capitalize string with control over whether to lower case the remainder."""
+    """Capitalize string with control over whether to lower case the remainder."""
     if lower_case_remainder:
         return x.capitalize()
     return x[0].upper() + x[1:]
@@ -30,10 +33,10 @@ def parameterize(word):
     return inflection.parameterize(word).replace("-", "_")
 
 
-def pluralize(word):
+def pluralize(word, count: str | int | None = None):
     if word.lower().split()[-1] in inflection.UNCOUNTABLES:
         return word
-    return inflection.pluralize(word)
+    return p.plural(word, count)
 
 
 def titleize(word):
